{% extends 'core_main_app/user/sharing/modals/single-link.html' %}

{% block modal_title %}Share PID{% endblock %}
{% block modal_id %}pid-sharing-modal{% endblock %}
{% block modal_input_id %}pid-sharing-link{% endblock %}
{% block modal_submit_id %}pid-sharing-submit{% endblock %}