{% extends 'core_main_app/user/sharing/button.html' %}

{% block btn_id %}pid-list-sharing{% endblock %}
{% block btn_class %}result-toolbar-button{% endblock %}
{% block btn_icon %}link{% endblock %}
{% block btn_name %}Share PIDs{% endblock %}
